C***********************************************************************
C                           UNIT INCOR                                 *
C               Previous modification: February 24, 1993               *
C                  1992-2007 by Vladimir I. Haltrin                   *
C       Last modification: June 1, 2007 by  Vladimir I. Haltrin        *
C***********************************************************************
      SUBROUTINE ELASTIC(ALFA,KB,MU,RS,MUS,LO,Z
     &                      ,EQD,ESD,ESU, dEQDdz,dESDdz,dESUdz, EKB)
C-----------------------------------------------------------------------
C     Actually we calculate here normalized irradiances EQD,ESD,ESU and
C     their derivatives dEQDdz,dESDdz,dESUdz [divided by EKB=exp(-nu*z)]
C     This is absolutly necessary in order to achieve proper accuracy in 
C     further calculations because EKB exponentially decays with depth.
C***********************************************************************
      IMPLICIT    none
      EXTERNAL    THXDVX,R10EXP
      REAL*8      ALFA,KB,MU,RS,MUS,LO,Z,EQD,ESD,ESU,dEQDdz,dESDdz,QL
      REAL*8      dESUdz,ET,M1,M2,FT,DO,DZ,QR,Q1,Q2,LM,NV,EKB,ZS,EZ,LQ
      REAL*8      R10EXP

        NV  = ALFA*KB
        ET  = ALFA/MUS
        M1  = 1.+MUS*(2.+MU)
        M2  = MUS*(2.-MU)-1.
        QR  = RS*ET*(.5+MU/(1.+MU*MU))
        Q1  = M1*QR
        Q2  = M2*QR
        EKB = R10EXP(-NV*Z)               ! < EKB is is our factor
        ZS   = .5*Z*(NV-ET)
        EZ = R10EXP(2*ZS)	  
        CALL THXDVX(ZS,FT)
        DO  = .5*Z*(1.+EZ)*FT
        DZ  = NV*DO-EZ 
        LM  = LO*MUS
C          Normalized downward irradiance of the source plus halo rays:
        EQD  = LM*EZ
      dEQDdz = EQD*ET   !  Its normalized negative derivative,-dEQD/dz
          LQ = LM*Q1
         ESD = LQ*DO    !  Downward scattered irradiance (without halo)
      dESDdz = LQ*DZ             !  Its negative derivative,-dESD/dz
          QL = LM*Q2
         ESU = LM*RS+QL*DO       !  Upward scattered elastic irradiance
      dESUdz = LM*NV*RS+QL*DZ    !  Its negative derivative,-dESU/dz

      RETURN
      END              !  of SUBROUTINE ELASTIC

C***********************************************************************
      SUBROUTINE RAMANIRR(BR,FR,NR,NDR, MUS,LO,HZ, NV,ET,TK,TN
     &                    ,ALFA,ROI,RBI,KOI,L,  XRD,XRU,dXRD,dXRU)
C-----------------------------------------------------------------------
C     This subroutine calculates Raman scattering corrections to  
C     irradiances ERD, ERU and their derivatives -dERD/dz and -dERU/dz
C     divided by K1EX=(k1/2)exp(-nu*z).
C     This is absolutly necessary in order to achieve proper accuracy in 
C     further calculations because K1EX exponentially decays with depth
C     and behaves like 1/<mu> at small <mu>, [HZ=Z/2].
C***********************************************************************
      IMPLICIT      none
      EXTERNAL      THXDVX,TXYSUB,R10EXP
      INTEGER*4     K,L,LA,M, NR,NDR
      REAL*8        BR(165),FR(NDR),NV(165),ET(165),TK(165),TN(165)
      REAL*8        LO(165),MUS,XRU,XRD,dXRU,dXRD,HZ,D3,TXY,R10EXP
      REAL*8        ALFA,ROI,RBI,KOI,DRL,K2,DRT,KSL,NUL,ETL,LN
      REAL*8        ETLA,ST,NULA,L1,L2,KT,NT,FT1,D1,FT2,E21,D2,FT3
      REAL*8        AVR,KDN,NKL,R1,R2,R3,E1,E2,E3,  XD,XU,dXD,dXU  !,AR
      EQUIVALENCE   (DRT,KDN),(DRL,NKL)

      NUL = NV(L)
      K2 = (1.+ROI)/(1.+RBI)
      KSL = ALFA*KOI
      ETL = ET(L)
      XD  = .0
      XU  = .0
      dXD = .0
      dXU = .0                   !  NR,  shift of Raman scattering band                
      M = L+NR-1-NDR/2           !  NDR, width of Raman scattering band
      Do K = 1, NDR          !  <-- Entry point to the integration loop
         LA  = K+M               !  Excitation wave number
         ETLA = ET(LA)
         L1  = 1./(KSL+ETLA)
         NULA = NV(LA)
         L2  = 1./(KSL+NULA)
         KT  = TK(LA)
         NT  = TN(LA)
         R1  = HZ*(NUL-ETLA)
         E1 = R10EXP(2.*R1)
         CALL THXDVX(R1,FT1)
         D1  = HZ*(1.+E1)*FT1
         R2  = HZ*(NUL-NULA)
         E2 = R10EXP(2.*R2)
         CALL THXDVX(R2,FT2)
         E21 = 1.+E2
         D2  = HZ*E21*FT2
         R3  = R2-R1
         E3 = R10EXP(2.*R3)
         CALL THXDVX(R3,FT3)
         D3  = HZ*(1.+E3)*FT3
         CALL TXYSUB(R1,R2,TXY)
         ST  = HZ*HZ*((E1+E2)*FT1*FT3-E21*TXY)
         LN  = L2*(NT+KT*L1)*E2+L1*(1+KT*D3)*E1
         AVR = BR(LA)*FR(K)*LO(LA)              !  was BR(L)
         DRT = K2*(D1+NT*D2+KT*ST) 
         DRL = DRT-ROI*(L1+L2*(NT+KT*L1))
		 XD  = XD+AVR*(DRL+ROI*LN)
         XU  = XU+AVR*(RBI*DRL+LN)
         KDN = (1.+KT*D3)*E1+NT*E2
         NKL = LN*(NUL+KSL)
         dXD = dXD+AVR*(KDN*(ROI-K2)-ROI*NKL)
         dXU = dXU+AVR*(KDN*(1.-RBI*K2)-NKL)
      EndDo                          !  <-- End of the integration loop
         XRD = XD
         XRU = XU
        dXRD = NUL*XD+dXD
        dXRU = NUL*XU+dXU

      RETURN
      END              !  of SUBROUTINE RAMANIRR

C***********************************************************************
      SUBROUTINE FLUORIRR(BI,MUS,LO,HZ,NV,ET,TK,TN,ALFA,ROI,RBI,KOI,L
     &                                            , XID,XIU,dXID,dXIU)
C-----------------------------------------------------------------------
C     This subroutine calculates fluorescent corrections
C     to irradiances EID, EIU and their derivatives -dEID/dz and
C      -dEIU/dz divided by K1EX=(k1/2)exp(-nu*z).
C     This is absolutly necessary in order to achieve proper accuracy 
C     in further calculations because K1EX exponentially decays with
C     depth and behaves like 1/<mu> at small <mu>, [HZ=Z/2].
C***********************************************************************
      IMPLICIT      none
      EXTERNAL      THXDVX,TXYSUB,R10EXP
      INTEGER*4     J,K,L
      REAL*8        BI(165),NV(165),ET(165),TK(165),TN(165)
      REAL*8        LO(165),MUS,XIU,XID,dXIU,dXID,HZ,D3,TXY,R10EXP
      REAL*8        ALFA,ROI,RBI,KOI,DRL,K2,DRT,KSL,NUL,ETL,LN
      REAL*8        ETLA,ST,NULA,L1,L2,KT,NT,FT1,D1,FT2,E21,D2,FT3
      REAL*8        AVR,KDN,NKL,R1,R2,R3,E1,E2,E3,  XD,XU,dXD,dXU

      NUL = NV(L)
      K2 = (1.+ROI)/(1.+RBI)
      KSL = ALFA*KOI
      ETL = ET(L)
      XD  = .0
      XU  = .0
      dXD = .0
      dXU = .0                   
      DO J = 1, 150      !  <-- Entry point to the integration loop
         K = J+14        !  K = 15,164 because BI(J14)=.0, BI(165)=.0
           ETLA = ET(K)
           L1  = 1./(KSL+ETLA)
           NULA = NV(K)
           L2  = 1./(KSL+NULA)
           KT  = TK(K)
           NT  = TN(K)
           R1  = HZ*(NUL-ETLA)
           E1 = R10EXP(2.*R1)
           CALL THXDVX(R1,FT1)
           D1  = HZ*(1.+E1)*FT1
           R2  = HZ*(NUL-NULA)
           E2 = R10EXP(2.*R2)
           CALL THXDVX(R2,FT2)
           E21 = 1.+E2
           D2  = HZ*E21*FT2
           R3  = R2-R1
           E3 = R10EXP(2.*R3)
           CALL THXDVX(R3,FT3)
           D3  = HZ*(1.+E3)*FT3
           CALL TXYSUB(R1,R2,TXY)
           ST  = HZ*HZ*((E1+E2)*FT1*FT3-E21*TXY)
           LN  = L2*(NT+KT*L1)*E2+L1*(1+KT*D3)*E1
           AVR = BI(K)*LO(K)
           DRT = K2*(D1+NT*D2+KT*ST) 
           DRL = DRT-ROI*(L1+L2*(NT+KT*L1))
		 XD  = XD+AVR*(DRL+ROI*LN)
         XU  = XU+AVR*(RBI*DRL+LN)
           KDN = (1.+KT*D3)*E1+NT*E2
           NKL = LN*(NUL+KSL)
         dXD = dXD+AVR*(KDN*(ROI-K2)-ROI*NKL)
         dXU = dXU+AVR*(KDN*(1.-RBI*K2)-NKL)
      ENDDO                          !  <-- End of the integration loop
         XID = XD
         XIU = XU
        dXID = NUL*XD+dXD
        dXIU = NUL*XU+dXU

      RETURN
      END              !  of SUBROUTINE FLUORIRR
	  
C***********************************************************************
      SUBROUTINE RAMANDRCOR(BR,FR,NR,NDR, MUS,LO, NV,ET,TK,TN
     &                                       ,ALFA,RB,KO,L, DRR)
C-----------------------------------------------------------------------
C     Raman scattering correction DRR to Diffuse Reflectance Rs
C***********************************************************************
      IMPLICIT     none
      INTEGER*4    K,L,LA,M, NR,NDR
      REAL*8       BR(165),FR(NR),LO(165),NV(165),ET(165),TK(165)
      REAL*8       TN(165), ALFA,DX,XRR,DRR,MUS,KO,RB,KSL,L1,L2,LM,AVR
      EQUIVALENCE  (LM,XRR)

      LM  = LO(L)*MUS
      DX  = 0.
      IF (LM.NE.0.) DX = (1.+RB)/(2.*LM)
      KSL = ALFA*KO
      XRR  = .0
      M = L+NR-1-NDR/2           !  NR, shift of Raman scattering band
      DO K = 1, NDR         !  <-- Entry point to the integration loop
         LA  = K+M               !  Excitation wave number
           L1 = 1./(KSL+ET(LA))
           L2 = 1./(KSL+NV(LA))
           AVR = BR(LA)*FR(K)*LO(LA)           !  was BR(L)
         XRR = XRR+AVR*(L1+L2*(TN(LA)+TK(LA)*L1))
      ENDDO                 !  <-- End of the integration loop
         DRR = DX*XRR

      RETURN
      END              !  of SUBROUTINE RAMANDRCOR

C***********************************************************************
      SUBROUTINE FLUORDRCOR(BI,MUS,LO,NV,ET,TK,TN,ALFA,RB,KO,L, DRI)
C-----------------------------------------------------------------------
C     Chlorophyll fluorescence correction DRC to Diffuse Reflectance Rs
C***********************************************************************
      IMPLICIT     none
      INTEGER*4    L, J,K
      REAL*8       BI(165),LO(165),NV(165),ET(165),TK(165)
      REAL*8       TN(165), ALFA,DX,XRI,DRI,MUS,KO,RB,KSL,LM,L1,L2

      LM  = LO(L)*MUS
      DX  = 0.
      IF (LM.NE.0.) DX = (1.+RB)/(2.*LM)
      KSL = ALFA*KO
      XRI  = .0
      DO J = 1, 150     !  <-- Entry point to the integration loop
         K = J+14       !  K = 15 ..164 because BI(J14)=.0, BI(165)=.0
           L1 = 1./(KSL+ET(K))
           L2 = 1./(KSL+NV(K))
         XRI  = XRI+BI(K)*LO(K)*(L1+L2*(TN(K)+TK(K)*L1))
      ENDDO                  !  <-- End of the integration loop
         DRI = DX*XRI

      RETURN
      END              !  of SUBROUTINE FLUORDRCOR
C***********************************************************************
