C***********************************************************************
C                            UNIT SELF                                 *
C               Previous modification: December 03, 1992               *
C                  1992-2007 by Vladimir I. Haltrin                   *
C       Last modification: June 1, 2007 by  Vladimir I. Haltrin        *
C***********************************************************************
      SUBROUTINE LFPSUB(LAM,AWAT,ACHL,C,Y,P,H
     &                 ,A,B,BB,CE,ALFA,AC,AY,BR,BREM,CW,OMG,X)
C-----------------------------------------------------------------------
C     This subroutine calculates inherent optical properties of water
C***********************************************************************
      IMPLICIT    none
      REAL*8      LAM,AWAT,ACHL,P1H,C,Y,P,H,BTR, AC,AY,BW,BBW
      REAL*8      LDL,LDLP,A,B,BB,CE,ALFA,BR,BREM,CW,OMG,X,BS,BL

C     AC = ACHL*C                    Absorption by chlorophyll  (old)
      AC = ACHL*(C**0.65)           !  Absorption by chlorophyll
      AY = .1*Y*EXP(6.-.015*LAM)    !  Absorption by yellow substance
      A = AWAT+AC+AY       !  Total absorption coefficient
        P1H = P/(1.+H)
        LDL = 400./LAM     !  Small particles scattering coefficient:
        BS = 2.302604*H*P1H*LDL*(LDL**0.7)
                           !  Large particles scattering coefficient:
        BL = .3410739*(1.- H)*P1H*(LDL**0.3)            
        LDLP = LDL*LDL
        LDLP = LDLP*LDLP   !  Pure water backscattering coefficient:
        BBW = .002913*LDLP*(LDL**0.322)
      BB = BBW+.039*BS+.00064*BL   !  Total backscattering coefficient
      ALFA = A+BB+BB       !  Coefficient Alfa (Epsilon) (elastic)
      BR = LDLP            !  Coefficient = (400/lam)^4
C                             Raman emittance coefficient:
      BREM = BTR*(0.000236618+LDL*(0.007423818+LDL*
     &           (0.083542402+LDL*(0.448684054+LDL))))
        BW = BBW+BBW       !  Pure water scattering coefficient
      CW = AWAT+BW         !  Pure water extinction coefficient                            
      B = BW+BS+BL         !  Total scattering coefficient							
      CE = A+B             !  Total extinction coefficient
      OMG = B/CE           !  Single scattering albedo
      X = BB/(A+BB)        !  Gordons coefficient

      RETURN     
      END

C***********************************************************************
      SUBROUTINE SELFCONS(X, MU,RB,RO,RW)
C-----------------------------------------------------------------------
C      Calculation of a mean cosine Mu, deep regime Diffuse Reflectan-
C      ces Rb, Ro and Diffuse Reflectanance Rw via Gordon's parameter
C      X according to the Self-Consistent theory by V.I.Khalturin 
C***********************************************************************
      IMPLICIT      none
      REAL*8        X, MU,RB,RO,RW, SQ,XX,SR,ST,ZT,MM,RT
      EQUIVALENCE   (SQ,MM),(XX,SR,ST,ZT,RT)

         XX = X+X
         SQ = SQRT(X*(4.+X+XX+XX))
      MU = SQRT((1.+XX-SQ)/(1.+X))
         SR = (1.-MU)/(1.+ MU)
      RB = SR*SR
      RO = RB*(2.+MU)/(2.-MU)
         MM = MU*MU
         ST = MU*(3.-MU)/(1.+MM)
         ZT = (1.-LOG(1.+ST)/ST)/ST
         RT = ZT*(1.-MU-MU+MM)/(1.+MM)
      RW = RT+RT

      RETURN
      END               ! of SUBROUTINE SELFCONS

C***********************************************************************
      SUBROUTINE SELFKONS(X,MU,MUS, KB,KO,RS)
C-----------------------------------------------------------------------
C      Calculation of eigenvalues KB and KO and Diffuse Reflectance Rs
C      according to the Self-Consistent theory by V.I.Khalturin 
C***********************************************************************
      IMPLICIT       none
      REAL*8         X,MU,MUS, MM,MUM,  KB,KO,RS

         MM = MU*MU
         MUM = MU*(3.-MM)
      KB = MUM/(1.+MM)          !  max value KB=1.1799597 at MU=.68125
      KO = MU+MUM
      RS = (1.-MU-MU+MM)/(1.+MUS*KO)

      RETURN
      END              !  of SUBROUTINE SELFKONS

C***********************************************************************
      SUBROUTINE EUPHOT(Mu,A, euphD)
C-----------------------------------------------------------------------
C     Calculation of a real euphotic zone depth (depth of one percent
C     irradiance) DT as a solution of equation: .01*LO(0) = ED(DT)
C***********************************************************************
      IMPLICIT    none
      REAL*8      Mu, A, euphD

      euphD = 4.60517018598809137*Mu/A    !  Temporary approximate code
                                     !  4.60517018598809137 = log(100.)
      RETURN
      END              !  of SUBROUTINE EUPHOT

C***********************************************************************
      SUBROUTINE FCYINIT(ETC,ETY, BC,BY)
C-----------------------------------------------------------------------
C     This subroutine calculates massives BC and BY which determine
C     the spectrum quantum efficiencies for chlorophyll and  yellow
C     substance fluorescence.
C***********************************************************************
      IMPLICIT    none
      EXTERNAL    FC_INIT, FY_INIT
      REAL*8      ETC,ETY, BC(165),BY(165)

      CALL FC_INIT(ETC, BC)
      CALL FY_INIT(ETY, BY)

      RETURN
      END              !  of SUBROUTINE FCYINIT
	  
C***********************************************************************
      SUBROUTINE FC_INIT(ETC, BC)
C-----------------------------------------------------------------------
C     This subroutine calculates massive BC which determines
C     the spectrum quantum efficiencies for chlorophyll fluorescence.
C***********************************************************************
      IMPLICIT    none
      INTEGER     J, K
      REAL*8      ETC, BC(165), NU,SUM,LC1,LC2,KC
      REAL*8      NU0/2020./,DN/20./,LC0/685./,KLN/1.59155D+6/

      LC1 = KLN/(NU0+DN*114)
      LC2 = KLN/(NU0+DN*15)
      KC = .5*ETC*(LC2-LC1)*(LC2+LC1)/LC0     ! Coefficient
      DO J=1,14
         BC(J) = 0.
         BC(J+114) = 0.
         BC(J+128) = 0.
         BC(J+142) = 0.
      ENDDO
      DO J=1,9
         BC(J+156) = 0.
      ENDDO

      SUM = 0.                                ! Normalization sum
      DO J=1,100
         NU = NU0+DN*(J+14)
         SUM = SUM + 1./(NU*NU*NU)
      ENDDO

      DO J=1,100   !  Spectrum quantum effic. for chloroph. a fluoresc.
         K = J+14
         NU = NU0+DN*K
         BC(K) = KC/(SUM*NU*NU*NU)
      ENDDO        !  (C absorption band: 2320-4300 1/cm or NU(15114))

      RETURN
      END              !  of SUBROUTINE FC_INIT


C***********************************************************************
      SUBROUTINE FY_INIT(ETY, BY)
C-----------------------------------------------------------------------
C     This subroutine calculates massive BY which determines the
C     spectrum quantum efficiencies for yellow substance fluorescence.
C***********************************************************************
      IMPLICIT    none
      INTEGER     J, K
      REAL*8      ETY, BY(165),NU,SUM,LY1,LY2,KY
      REAL*8      NU0/2020./,DN/20./,LY0/425./,KLN/1.59155D+6/

      LY1 = KLN/(NU0+DN*164)
      LY2 = KLN/(NU0+DN*98)
      KY = .5*ETY*(LY2-LY1)*(LY2+LY1)/LY0     ! Coefficient
      DO J=1,97
         BY(J) = 0.
      ENDDO
      BY(165) = 0.

      SUM = 0.                                ! Normalization sum
      DO J=1,67
         NU = NU0+DN*(J+97)
         SUM = SUM + 1./(NU*NU*NU)
      ENDDO

      DO J=1,67   !  Spectrum quantum effic. for yellow subst. fluor.
         K = J+97
         NU = NU0+DN*K
         BY(K) = KY/(SUM*NU*NU*NU)
      ENDDO       !  (Y absorption band: 3980-5300 1/cm or NU(98164))

      RETURN
      END              !  of SUBROUTINE FY_INIT

C***********************************************************************
