C***********************************************************************
C                            UNIT XYSB                                 *
C               Previous modification: February 24, 1993               *
C                  1992-2007 by Vladimir I. Haltrin                   *
C       Last modification: June 1, 2007 by  Vladimir I. Haltrin        *
C-----------------------------------------------------------------------
C     A set of subroutines developed in order to resolve
C     uncertainties of the division by zero type.
C
C***********************************************************************
      SUBROUTINE THXDVX(X,Y)
C-----------------------------------------------------------------------
C     Calculation of Y = th(x)/x.
C***********************************************************************
      IMPLICIT    none
      EXTERNAL	THSML
      REAL*8      X,Y,THSML

      IF (ABS(X).LE.0.1) THEN
        Y = THSML(X)
      ELSE
        Y = TANH(X)/X
      ENDIF

      RETURN
      END

C***********************************************************************
      SUBROUTINE TXYSUB(X,Y,Z)
C-----------------------------------------------------------------------
C   Calculation of Z(x,y)=(th(x)/x-th(y)/y)/(y-x).
C***********************************************************************
      IMPLICIT     none
      EXTERNAL	XBGYBG,XBGYSM,FXAYSM,FXMYSM
      REAL*8       RD, XU,XD,AX, X,Y,Z   !  ,R
      PARAMETER    (RD = .1)

      IF (Y.LE.0) THEN
         X = -X                 !  The function Z(x,y) = Z(-x,-y),
         Y = -Y                 !  so we can reduce our code
      ENDIF
      AX = ABS(X)

      XU = X+RD
      XD = X-RD

      IF (X.LE.RD) THEN
         IF ((AX.GT.RD).AND.(Y.GT.RD)) THEN
            CALL XBGYBG(X,Y,Z)                           !  Region # 1a
            RETURN
         ELSEIF ((AX.GT.RD).AND.(Y.LE.RD)) THEN
            CALL XBGYSM(X,Y,Z)                           !  Region # 2a
            RETURN
         ELSEIF ((AX.LE.RD).AND.(Y.GT.RD).AND.(Y.GT.XU)) THEN
            CALL XBGYSM(Y,X,Z)                           !  Region # 2b
            RETURN
         ELSEIF ((AX.LE.RD).AND.(Y.LE.RD)) THEN
            CALL FXAYSM(X,Y,Z)                            !  Region # 3
            RETURN
         ELSE
            CALL FXMYSM(X,Y,Z)                           !  Region # 4a
            RETURN
         ENDIF
      ELSE
         IF ((Y.GT.XU).OR.((Y.LT.XD).AND.(Y.GT.RD))) THEN
            CALL XBGYBG(X,Y,Z)                   !  Regions # 1b and 1c
            RETURN
         ELSEIF ((Y.LE.RD).AND.(Y.LE.XD)) THEN
            CALL XBGYSM(X,Y,Z)                           !  Region # 2c
            RETURN
         ELSE
            CALL FXMYSM(X,Y,Z)                           !  Region # 4b
            RETURN
         ENDIF
      ENDIF

      RETURN
      END

C***********************************************************************
      SUBROUTINE XBGYBG(X,Y,Z)         ! X<0, Y>RD, |X|>RD
C***********************************************************************
      IMPLICIT     none
      REAL*8       X,Y,Z

      Z = (TANH(X)/X-TANH(Y)/Y)/(Y-X)

      RETURN
      END

C***********************************************************************
      SUBROUTINE XBGYSM(X,Y,Z)         ! X>RD, Y<=RD
C***********************************************************************
      IMPLICIT     none
      EXTERNAL	   THSML
      REAL*8       X,Y,Z,THSML

      Z = (TANH(X)/X-THSML(Y))/(Y-X)

      RETURN
      END

C***********************************************************************
      SUBROUTINE FXAYSM(X,Y,Z)         ! |X|<=RD, 0<=Y<=RD
C***********************************************************************
      IMPLICIT     none
      REAL*8       X,Y,Z,XX,YY,XY,TR,XPY,DXY,XR,ZM

        XX = X*X
        YY = Y*Y
        XY = X*Y
        TR = XX*X+YY*Y
        XPY = X+Y
        DXY = XX+YY
        XR = .053968254*(DXY+XY)*TR
        ZM = .021869489*(TR+XY*XPY)*(XX*XX+YY*YY)

      Z = XPY*(.333333333-.133333333*DXY)+XR-ZM

      RETURN
      END

C***********************************************************************
      SUBROUTINE FXMYSM(X,Y,Z)         ! X>RD, Y>RD, |X-Y|<RD
C-----------------------------------------------------------------------
C     Calculation of (th(X)/X-th(Y)/Y)/(Y-X) when |Y-X|<RD
C***********************************************************************
      IMPLICIT     none
      EXTERNAL	   THSML
      REAL*8       X,Y,Z,T,Y_X,TE,THSML

        Y_X = Y-X
        T  = TANH(X)
        TE = THSML(Y_X)
      Z = (T/X-(1.-T*T)*TE/(1.+T*Y_X*TE))/Y

      RETURN
      END

C***********************************************************************
      FUNCTION THSML(X)        ! Calculation of th(X)/X at |X|<=RD
C***********************************************************************
      IMPLICIT   NONE
      INTEGER*4  J
      REAL*8     THSML,X,Y,X2,X2N, C(12)
     &      / -0.33333333333333333333,     0.133333333333333333333,
     &        -0.053968253968253968254,    0.0218694885361552028219,
     &        -0.0088632355299021965689,   0.0035921280365724810169,
     &        -0.00145583438705131826825,  0.00059002744094558598138,
     &        -0.000239129114243552481486, 0.000096915379569294503256,
     &        -0.000039278323883316834053, 0.0000159189050693289647407/
C-----------------------------------------------------------------------
      Y = 1.
      X2 = X*X
      X2N = 1.
      DO J=1, 12
         X2N = X2N*X2
         Y = Y + C(J)*X2N
      END DO
      THSML = Y

      RETURN
      END

C***********************************************************************
      FUNCTION R10EXP(X)
C-----------------------------------------------------------------------
C     Calculation of exp(x)
C***********************************************************************
      IMPLICIT    none
      REAL*8      X,R10EXP
      REAL*8     XX

      XX = X
      R10EXP = DEXP(XX)

      RETURN
      END
C***********************************************************************
